package com.pfc.src.loginpreference;

import java.util.HashMap;

import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;

import com.pfc.src.LoginActivity;
 
public class SessionManager {
    // Shared Preferences
    SharedPreferences pref;
 
    // Editor for Shared preferences
    Editor editor;
 
    // Context
    Context _context;
 
    // Shared pref mode
    int PRIVATE_MODE = 0;
 
    // Sharedpref file name
    private static final String PREF_NAME = "AndroidHivePref";
 
    // All Shared Preferences Keys
    private static final String IS_LOGIN = "IsLoggedIn";
 
    // User name (make variable public to access from outside)
    public static final String KEY_NAME = "name";
 
    // Email address (make variable public to access from outside)
    public static final String KEY_EMAIL = "email";
 
    // Constructor
    public SessionManager(Context context){
        this._context = context;
        pref = _context.getSharedPreferences(PREF_NAME, PRIVATE_MODE);
        editor = pref.edit();
    }
 
    /**
     * Creaci de login session
     * */
    public void createLoginSession(String name, String email){
        // Gruarden el valor de login com a true
        editor.putBoolean(IS_LOGIN, true);
        // Guarden el nom a preferencies
        editor.putString(KEY_NAME, name);
        // Guardem el email dusuari a preferencies
        // Storing email in pref
        editor.putString(KEY_EMAIL, email);
 
        // validem els cambis
        editor.commit();
    }   
 
    /**
     * Checkea el estatus de login del usuario
     * Si es fals se redirecciona directament a la pgina de loggeig
     * Sino no fa res
     * */
    public void checkLogin(){
        // verifica el loggin status
        if(!this.isLoggedIn()){
            // Es redirecciona a la activitat de login
            Intent i = new Intent(_context, LoginActivity.class);
            // Es tancan totes les activitats
            i.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
 
            // Afegeix un now flag para comenar nova activitat
            i.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
 
            _context.startActivity(i);
        }
 
    }
 
    /**
     * Obte les dades de session guardades
     * */
    public HashMap<String, String> getUserDetails(){
        HashMap<String, String> user = new HashMap<String, String>();
        // nom deusuari
        user.put(KEY_NAME, pref.getString(KEY_NAME, null));
        // email del usuari
        user.put(KEY_EMAIL, pref.getString(KEY_EMAIL, null));
        // retorna el usuari
        return user;
    }
 
    /**
     * Clear session details
     * */
    public void logoutUser(){
        // Clearing all data from Shared Preferences
        editor.clear();
        editor.commit();
 
        // After logout redirect user to Loing Activity
        Intent i = new Intent(_context, LoginActivity.class);
        // Closing all the Activities
        i.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
 
        // Add new Flag to start new Activity
        i.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
 
        // Staring Login Activity
        _context.startActivity(i);
    }
 
    /**
     * Quick check for login
     * **/
    // Get Login State
    public boolean isLoggedIn(){
        return pref.getBoolean(IS_LOGIN, false);
    }
}
